package com.qzing.ime.abnl;

import com.qzing.framework.core.web.GlobalResponse;
import com.qzing.framework.core.web.PageBeanResponse;
import com.qzing.ieep.data.common.RestResponse;
import com.qzing.ieep.data.jpa.util.DataUtils;
import com.qzing.ime.framework.dto.BaseBatchExtraResponse;
import com.qzing.ime.framework.rest.GlobalHandlerProperties;
import com.qzing.ime.framework.toolkit.utils.ImeI18NUtils;
import com.qzing.orchestration.vo.OrchestrationResult;
import com.qzing.toolkit.spring.api.QZingController;
import com.qzing.ui.utils.PathPatternUtil;
import com.qzing.ui.web.UiGlobalResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.Order;
import org.springframework.http.MediaType;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/**
 * 全局返回结果处理器
 *
 * <p>
 * 针对RestController和Controller控制器的非GlobalResponse全局返回结果进行统一拦截处理封装。</br>
 * 针对spring-data、mybatisplus的分页对象返回进行统一的PageBean封装并返回全局结果。</br>
 * </p>
 */
@Slf4j
@ControllerAdvice(annotations = {RestController.class, Controller.class, QZingController.class})
@Order(10000)
@EnableConfigurationProperties({GlobalHandlerProperties.class})
public class AbnlGlobalResponseHandler implements ResponseBodyAdvice {

    @Autowired
    private GlobalHandlerProperties globalHandlerProperties;

    @Override
    public boolean supports(MethodParameter returnType, Class converterType) {
        return true;
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType,
                                  Class selectedConverterType, ServerHttpRequest request, ServerHttpResponse response) {

        // 排除指定url的全局响应处理
        String path = request.getURI().getPath();
        Collection<String> excludeUrls = new HashSet<>();
        if (globalHandlerProperties != null) {
            excludeUrls.addAll(globalHandlerProperties.getResponseHandlerExcludeUrls());
        }
        if (excludeUrls.size() > 0 && PathPatternUtil.matches(excludeUrls, path)) {
            return body;
        }

        Map<String, Object> extra = new HashMap<>();

        if (body instanceof RestResponse) {
            return body;
        }
        if (body instanceof GlobalResponse) {
            return body;
        }
        if (body instanceof UiGlobalResponse) {
            return body;
        }
        if (body instanceof BaseBatchExtraResponse) {
            extra.putAll(DataUtils.parseMap(body));
            return RestResponse.success(null, extra);
        }
        if (body instanceof OrchestrationResult) {
            OrchestrationResult result = (OrchestrationResult) body;
            if (result != null) {
                Map<String, Object> bpmExtra = result.getExtra();
                if (bpmExtra != null) {
                    Boolean imeException = (Boolean) bpmExtra.get("imeException");
                    if (imeException != null && imeException) {
                        result.setSuccess(false);
                        String imeExceptionCode = (String) bpmExtra.get("imeExceptionCode");
                        String imeExceptionMessage = bpmExtra.get("imeExceptionMessage") == null ? "执行编排位置异常，请查看编排实例ID" + result.getInstanceId() : (String) bpmExtra.get("imeExceptionMessage");
                        if (!StringUtils.isEmpty(imeExceptionCode)) {
                            result.setCode(imeExceptionCode);
                        }
                        if (!StringUtils.isEmpty(imeExceptionMessage)) {
                            result.setMessage(imeExceptionMessage);
                        }
                    }
                }
            }
            return result;
        }

        final String returnTypeName = returnType.getParameterType().getName();
        // void直接返回
        if ("void".equals(returnTypeName)) {
            return RestResponse.success(null);
        }
        final Class<?> returnTypeClass = returnType.getParameterType();
        if (PageBeanResponse.class.isAssignableFrom(returnTypeClass)) {
            return body;
        }
        return RestResponse.success(null, body);
    }

    private String getActionI18nByPath(String path) {
        if (path.endsWith("/")) {
            path = path.substring(path.length() - 1);
        }
        //将/equip/equip/inspectstandard/save 换成/ime/equip/equip/save
        //第一固定换成ime
        int secondIndex = path.indexOf("/", path.indexOf("/") + 1);
        path = "/ime" + path.substring(secondIndex);
        //将/替换成为.
        String i18nKey = path.substring(1).replace('/', '.') + ".action";
        return ImeI18NUtils.getTextWithReturnNull(i18nKey);
    }
}
